<?php

namespace App\Http\Controllers\Admin;

use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Services\AdminService;
use Illuminate\Support\Facades\DB;
use App\Http\Resources\DefaultResource;
use App\Http\Controllers\MainApiController;
use App\Http\Requests\VerifyActivationCodeRequest;
use App\Http\Requests\Password\{ResetPassword, SetNewPasswordRequest, ChangePasswordRequest};
use App\Http\Requests\UpdateAdminProfileRequest;
use App\Http\Requests\Admin\AdminLoginRequest;
use App\Models\User;
use App\Http\Resources\Admin\ProfileResource;

class AuthController extends MainApiController
{
    private $adminService;

    public function __construct(AdminService $adminService)
    {
        parent::__construct();
        $this->adminService = $adminService;
    }


    public function update(UpdateAdminProfileRequest $request)
    {
        auth()->user()->first_name = $request->first_name;
        auth()->user()->last_name = "";
        // auth()->user()->phone = $request->phone;

        if ($request->has('photo')) {

            $saveFile = $this->saveFile($request->photo, User::IMAGES_DIRECTORY, 'local');
            auth()->user()->photo = $saveFile['fileName'];
        }

        auth()->user()->save();

        return response()->json(['status' => true, 'message' => 'Profile Updated Successfully', 'data' => new ProfileResource(auth()->user())]);
    }


    public function updatePassword(ChangePasswordRequest $request)
    {
        $user = $this->adminService->updatePassword($request, auth()->user()->id);

        return $this->response->success(
            new DefaultResource(['message' => 'Password changed successfully!'])
        );
    }



    public function login(AdminLoginRequest $request)
    {

        $data = $this->adminService->checkLogin($request, 'admin');

        return response()->json($data);
    }

    public function reset(ResetPassword $request)
    {
        return $this->adminService->reset($request);
    }

    public function setPassword(SetNewPasswordRequest $request)
    {
        return $this->adminService->verifyAndUpdatePassword($request);
    }

    public function verifyCode(VerifyActivationCodeRequest $request)
    {
        return response()->json(['success' => true, 'message' => 'Code verified']);
    }

    public function sendVerificationCode(ResetPassword $request)
    {

        $user = $this->adminService->getUserByEmail($request->email);
        return $this->adminService->sendAccountVerification($user);
    }

    public function logout(Request $request)
    {
        if (auth('sanctum')->check()) {
            auth('sanctum')->user()->currentAccessToken()->delete();
            return response()->json(['status' => true, 'message' => 'Logout Successful']);
        } else {
            return response()->json(['status' => false, 'message' => 'Invalid request'], 401);
        }
    }

    public function profile(Request $request)
    {

        $admin = auth()->guard()->user();
        return response()->json(['status' => true, 'data' => new ProfileResource($admin)]);
    }

    public function dashboardStats(Request $request)
    {

        $stats = $this->adminService->stats();

        return response()->json($stats);
    }

    public function dashboardGraph(Request $request)
    {
        $graphData = $this->adminService->graphData($request);

        return response()->json($graphData);
    }

    public function postStats(Request $request)
    {
        $monthlyTrend = User::selectRaw('COUNT(id) as count')->whereRaw("created_at LIKE '" . date('Y-m') . "%'")->first();
        $totalPosts = User::count();

        return response()->json(['status' => true, 'total_posts' => $totalPosts->count, 'monthly_trends' => $monthlyTrend->count]);
    }

    public function chartData(Request $request)
    {
        $data = User::select(
            DB::raw("(COUNT(*)) as count"),
            DB::raw("MONTHNAME(created_at) as month_name")
        )
            ->whereYear('created_at', date('Y'))
            ->groupBy('month_name')
            ->get();
        $months = $records = [];
        foreach ($data as $dt) {
            $months[] = $dt['month_name'];
            $records[] = $dt['count'];
        }


        return response()->json(['status' => true, 'data' => $data, 'months' => $months, 'records' => $records]);
    }

    public function filterChartData(Request $request)
    {

        $filter = request()->input('filter');
        $dataQuery = User::select(
            DB::raw("(COUNT(*)) as count"),
            DB::raw("MONTHNAME(created_at) as month_name")
        )
            ->groupBy('month_name');

        if ($filter == "Y") {
            $data = $dataQuery->whereYear('created_at', date('Y'))->get();
        } elseif ($filter == "M") {
            $data = $dataQuery->whereMonth('created_at', date('m'))->get();
        } elseif ($filter == "Q") {
            $data = $dataQuery->whereBetween('created_at', [Carbon::now()->subMonths(3), Carbon::now()->endOfMonth()])->get();
        }

        $months = $records = [];
        foreach ($data as $dt) {
            $months[] = $dt['month_name'];
            $records[] = $dt['count'];
        }


        return response()->json(['status' => true, 'data' => $data, 'months' => $months, 'records' => $records]);
    }
}
